@@ -1,9 +1,9 @@
 use std::env;
 use std::fmt::{Debug, Formatter};
 use std::str::FromStr;
-use std::time::Duration;
 
 use http::{HeaderMap, HeaderName, HeaderValue};
+use opentelemetry::otel_debug;
 use tonic::codec::CompressionEncoding;
 use tonic::metadata::{KeyAndValueRef, MetadataMap};
 use tonic::service::Interceptor;
@@ -12,20 +12,21 @@ use tonic::transport::Channel;
 use tonic::transport::ClientTlsConfig;
 
 use super::{default_headers, parse_header_string, OTEL_EXPORTER_OTLP_GRPC_ENDPOINT_DEFAULT};
+use super::{resolve_timeout, ExporterBuildError};
 use crate::exporter::Compression;
 use crate::{
     ExportConfig, OTEL_EXPORTER_OTLP_COMPRESSION, OTEL_EXPORTER_OTLP_ENDPOINT,
-    OTEL_EXPORTER_OTLP_HEADERS, OTEL_EXPORTER_OTLP_TIMEOUT,
+    OTEL_EXPORTER_OTLP_HEADERS,
 };
 
 #[cfg(feature = "logs")]
-mod logs;
+pub(crate) mod logs;
 
 #[cfg(feature = "metrics")]
-mod metrics;
+pub(crate) mod metrics;
 
 #[cfg(feature = "trace")]
-mod trace;
+pub(crate) mod trace;
 
 /// Configuration for [tonic]
 ///
@@ -34,32 +35,32 @@ mod trace;
 #[non_exhaustive]
 pub struct TonicConfig {
     /// Custom metadata entries to send to the collector.
-    pub metadata: Option<MetadataMap>,
-
+    pub(crate) metadata: Option<MetadataMap>,
     /// TLS settings for the collector endpoint.
     #[cfg(feature = "tls")]
-    pub tls_config: Option<ClientTlsConfig>,
-
+    pub(crate) tls_config: Option<ClientTlsConfig>,
     /// The compression algorithm to use when communicating with the collector.
-    pub compression: Option<Compression>,
+    pub(crate) compression: Option<Compression>,
+    pub(crate) channel: Option<tonic::transport::Channel>,
+    pub(crate) interceptor: Option<BoxInterceptor>,
 }
 
 impl TryFrom<Compression> for tonic::codec::CompressionEncoding {
-    type Error = crate::Error;
+    type Error = ExporterBuildError;
 
-    fn try_from(value: Compression) -> Result<Self, Self::Error> {
+    fn try_from(value: Compression) -> Result<Self, ExporterBuildError> {
         match value {
             #[cfg(feature = "gzip-tonic")]
             Compression::Gzip => Ok(tonic::codec::CompressionEncoding::Gzip),
             #[cfg(not(feature = "gzip-tonic"))]
-            Compression::Gzip => Err(crate::Error::FeatureRequiredForCompressionAlgorithm(
+            Compression::Gzip => Err(ExporterBuildError::FeatureRequiredForCompressionAlgorithm(
                 "gzip-tonic",
                 Compression::Gzip,
             )),
             #[cfg(feature = "zstd-tonic")]
             Compression::Zstd => Ok(tonic::codec::CompressionEncoding::Zstd),
             #[cfg(not(feature = "zstd-tonic"))]
-            Compression::Zstd => Err(crate::Error::FeatureRequiredForCompressionAlgorithm(
+            Compression::Zstd => Err(ExporterBuildError::FeatureRequiredForCompressionAlgorithm(
                 "zstd-tonic",
                 Compression::Zstd,
             )),
@@ -67,21 +68,6 @@ impl TryFrom<Compression> for tonic::codec::CompressionEncoding {
     }
 }
 
-fn resolve_compression(
-    tonic_config: &TonicConfig,
-    env_override: &str,
-) -> Result<Option<CompressionEncoding>, crate::Error> {
-    if let Some(compression) = tonic_config.compression {
-        Ok(Some(compression.try_into()?))
-    } else if let Ok(compression) = env::var(env_override) {
-        Ok(Some(compression.parse::<Compression>()?.try_into()?))
-    } else if let Ok(compression) = env::var(OTEL_EXPORTER_OTLP_COMPRESSION) {
-        Ok(Some(compression.parse::<Compression>()?.try_into()?))
-    } else {
-        Ok(None)
-    }
-}
-
 /// Configuration for the [tonic] OTLP GRPC exporter.
 ///
 /// It allows you to
@@ -96,36 +82,30 @@ fn resolve_compression(
 ///
 /// ```no_run
 /// # #[cfg(feature="metrics")]
-/// use opentelemetry_sdk::metrics::reader::{
-///     DefaultAggregationSelector, DefaultTemporalitySelector,
-/// };
+/// use opentelemetry_sdk::metrics::Temporality;
 ///
 /// # fn main() -> Result<(), Box<dyn std::error::Error>> {
 /// // Create a span exporter you can use to when configuring tracer providers
 /// # #[cfg(feature="trace")]
-/// let span_exporter = opentelemetry_otlp::new_exporter().tonic().build_span_exporter()?;
+/// let span_exporter = opentelemetry_otlp::SpanExporter::builder().with_tonic().build()?;
 ///
-/// // Create a metrics exporter you can use when configuring meter providers
+/// // Create a metric exporter you can use when configuring meter providers
 /// # #[cfg(feature="metrics")]
-/// let metrics_exporter = opentelemetry_otlp::new_exporter()
-///     .tonic()
-///     .build_metrics_exporter(
-///         Box::new(DefaultAggregationSelector::new()),
-///         Box::new(DefaultTemporalitySelector::new()),
-///     )?;
+/// let metric_exporter = opentelemetry_otlp::MetricExporter::builder()
+///     .with_tonic()
+///     .with_temporality(Temporality::default())
+///     .build()?;
 ///
 /// // Create a log exporter you can use when configuring logger providers
 /// # #[cfg(feature="logs")]
-/// let log_exporter = opentelemetry_otlp::new_exporter().tonic().build_log_exporter()?;
+/// let log_exporter = opentelemetry_otlp::LogExporter::builder().with_tonic().build()?;
 /// # Ok(())
 /// # }
 /// ```
 #[derive(Debug)]
 pub struct TonicExporterBuilder {
-    pub(crate) exporter_config: ExportConfig,
     pub(crate) tonic_config: TonicConfig,
-    pub(crate) channel: Option<tonic::transport::Channel>,
-    pub(crate) interceptor: Option<BoxInterceptor>,
+    pub(crate) exporter_config: ExportConfig,
 }
 
 pub(crate) struct BoxInterceptor(Box<dyn Interceptor + Send + Sync>);
@@ -143,93 +123,40 @@ impl Debug for BoxInterceptor {
 
 impl Default for TonicExporterBuilder {
     fn default() -> Self {
-        let tonic_config = TonicConfig {
-            metadata: Some(MetadataMap::from_headers(
-                (&default_headers())
-                    .try_into()
-                    .expect("Invalid tonic headers"),
-            )),
-            #[cfg(feature = "tls")]
-            tls_config: None,
-            compression: None,
-        };
-
         TonicExporterBuilder {
+            tonic_config: TonicConfig {
+                metadata: Some(MetadataMap::from_headers(
+                    (&default_headers())
+                        .try_into()
+                        .expect("Invalid tonic headers"),
+                )),
+                #[cfg(feature = "tls")]
+                tls_config: None,
+                compression: None,
+                channel: Option::default(),
+                interceptor: Option::default(),
+            },
             exporter_config: ExportConfig {
                 protocol: crate::Protocol::Grpc,
                 ..Default::default()
             },
-            tonic_config,
-            channel: Option::default(),
-            interceptor: Option::default(),
         }
     }
 }
 
 impl TonicExporterBuilder {
-    /// Set the TLS settings for the collector endpoint.
-    #[cfg(feature = "tls")]
-    pub fn with_tls_config(mut self, tls_config: ClientTlsConfig) -> Self {
-        self.tonic_config.tls_config = Some(tls_config);
-        self
-    }
-
-    /// Set custom metadata entries to send to the collector.
-    pub fn with_metadata(mut self, metadata: MetadataMap) -> Self {
-        // extending metadata maps is harder than just casting back/forth
-        let incoming_headers = metadata.into_headers();
-        let mut existing_headers = self
-            .tonic_config
-            .metadata
-            .unwrap_or_default()
-            .into_headers();
-        existing_headers.extend(incoming_headers);
-
-        self.tonic_config.metadata = Some(MetadataMap::from_headers(existing_headers));
-        self
-    }
-
-    /// Set the compression algorithm to use when communicating with the collector.
-    pub fn with_compression(mut self, compression: Compression) -> Self {
-        self.tonic_config.compression = Some(compression);
-        self
-    }
-
-    /// Use `channel` as tonic's transport channel.
-    /// this will override tls config and should only be used
-    /// when working with non-HTTP transports.
-    ///
-    /// Users MUST make sure the [`ExportConfig::timeout`] is
-    /// the same as the channel's timeout.
-    pub fn with_channel(mut self, channel: tonic::transport::Channel) -> Self {
-        self.channel = Some(channel);
-        self
-    }
-
-    /// Use a custom `interceptor` to modify each outbound request.
-    /// this can be used to modify the grpc metadata, for example
-    /// to inject auth tokens.
-    pub fn with_interceptor<I>(mut self, interceptor: I) -> Self
-    where
-        I: tonic::service::Interceptor + Clone + Send + Sync + 'static,
-    {
-        self.interceptor = Some(BoxInterceptor(Box::new(interceptor)));
-        self
-    }
-
     fn build_channel(
         self,
         signal_endpoint_var: &str,
         signal_timeout_var: &str,
         signal_compression_var: &str,
         signal_headers_var: &str,
-    ) -> Result<(Channel, BoxInterceptor, Option<CompressionEncoding>), crate::Error> {
-        let tonic_config = self.tonic_config;
-        let compression = resolve_compression(&tonic_config, signal_compression_var)?;
+    ) -> Result<(Channel, BoxInterceptor, Option<CompressionEncoding>), ExporterBuildError> {
+        let compression = self.resolve_compression(signal_compression_var)?;
 
-        let headers_from_env = parse_headers_from_env(signal_headers_var);
+        let (headers_from_env, headers_for_logging) = parse_headers_from_env(signal_headers_var);
         let metadata = merge_metadata_with_headers_from_env(
-            tonic_config.metadata.unwrap_or_default(),
+            self.tonic_config.metadata.unwrap_or_default(),
             headers_from_env,
         );
 
@@ -248,7 +175,7 @@ impl TonicExporterBuilder {
             Ok(req)
         };
 
-        let interceptor = match self.interceptor {
+        let interceptor = match self.tonic_config.interceptor {
             Some(mut interceptor) => {
                 BoxInterceptor(Box::new(move |req| interceptor.call(add_metadata(req)?)))
             }
@@ -256,48 +183,26 @@ impl TonicExporterBuilder {
         };
 
         // If a custom channel was provided, use that channel instead of creating one
-        if let Some(channel) = self.channel {
+        if let Some(channel) = self.tonic_config.channel {
             return Ok((channel, interceptor, compression));
         }
 
         let config = self.exporter_config;
 
-        // resolving endpoint string
-        // grpc doesn't have a "path" like http(See https://github.com/grpc/grpc/blob/master/doc/PROTOCOL-HTTP2.md)
-        // the path of grpc calls are based on the protobuf service definition
-        // so we won't append one for default grpc endpoints
-        // If users for some reason want to use a custom path, they can use env var or builder to pass it
-        let endpoint = match env::var(signal_endpoint_var)
-            .ok()
-            .or(env::var(OTEL_EXPORTER_OTLP_ENDPOINT).ok())
-        {
-            Some(val) => val,
-            None => {
-                if config.endpoint.is_empty() {
-                    OTEL_EXPORTER_OTLP_GRPC_ENDPOINT_DEFAULT.to_string()
-                } else {
-                    config.endpoint
-                }
-            }
-        };
+        let endpoint = Self::resolve_endpoint(signal_endpoint_var, config.endpoint);
 
-        let endpoint = Channel::from_shared(endpoint).map_err(crate::Error::from)?;
-        let timeout = match env::var(signal_timeout_var)
-            .ok()
-            .or(env::var(OTEL_EXPORTER_OTLP_TIMEOUT).ok())
-        {
-            Some(val) => match val.parse() {
-                Ok(seconds) => Duration::from_secs(seconds),
-                Err(_) => config.timeout,
-            },
-            None => config.timeout,
-        };
+        // Used for logging the endpoint
+        let endpoint_clone = endpoint.clone();
+
+        let endpoint = Channel::from_shared(endpoint)
+            .map_err(|op| ExporterBuildError::InvalidUri(endpoint_clone.clone(), op.to_string()))?;
+        let timeout = resolve_timeout(signal_timeout_var, config.timeout.as_ref());
 
         #[cfg(feature = "tls")]
-        let channel = match tonic_config.tls_config {
+        let channel = match self.tonic_config.tls_config {
             Some(tls_config) => endpoint
                 .tls_config(tls_config)
-                .map_err(crate::Error::from)?,
+                .map_err(|er| ExporterBuildError::InternalFailure(er.to_string()))?,
             None => endpoint,
         }
         .timeout(timeout)
@@ -306,16 +211,51 @@ impl TonicExporterBuilder {
         #[cfg(not(feature = "tls"))]
         let channel = endpoint.timeout(timeout).connect_lazy();
 
+        otel_debug!(name: "TonicChannelBuilt", endpoint = endpoint_clone, timeout_in_millisecs = timeout.as_millis(), compression = format!("{:?}", compression), headers = format!("{:?}", headers_for_logging));
         Ok((channel, interceptor, compression))
     }
 
+    fn resolve_endpoint(default_endpoint_var: &str, provided_endpoint: Option<String>) -> String {
+        // resolving endpoint string
+        // grpc doesn't have a "path" like http(See https://github.com/grpc/grpc/blob/master/doc/PROTOCOL-HTTP2.md)
+        // the path of grpc calls are based on the protobuf service definition
+        // so we won't append one for default grpc endpoints
+        // If users for some reason want to use a custom path, they can use env var or builder to pass it
+        //
+        // programmatic configuration overrides any value set via environment variables
+        if let Some(endpoint) = provided_endpoint {
+            endpoint
+        } else if let Ok(endpoint) = env::var(default_endpoint_var) {
+            endpoint
+        } else if let Ok(endpoint) = env::var(OTEL_EXPORTER_OTLP_ENDPOINT) {
+            endpoint
+        } else {
+            OTEL_EXPORTER_OTLP_GRPC_ENDPOINT_DEFAULT.to_string()
+        }
+    }
+
+    fn resolve_compression(
+        &self,
+        env_override: &str,
+    ) -> Result<Option<CompressionEncoding>, ExporterBuildError> {
+        if let Some(compression) = self.tonic_config.compression {
+            Ok(Some(compression.try_into()?))
+        } else if let Ok(compression) = env::var(env_override) {
+            Ok(Some(compression.parse::<Compression>()?.try_into()?))
+        } else if let Ok(compression) = env::var(OTEL_EXPORTER_OTLP_COMPRESSION) {
+            Ok(Some(compression.parse::<Compression>()?.try_into()?))
+        } else {
+            Ok(None)
+        }
+    }
+
     /// Build a new tonic log exporter
     #[cfg(feature = "logs")]
-    pub fn build_log_exporter(
-        self,
-    ) -> Result<crate::logs::LogExporter, opentelemetry::logs::LogError> {
+    pub(crate) fn build_log_exporter(self) -> Result<crate::logs::LogExporter, ExporterBuildError> {
         use crate::exporter::tonic::logs::TonicLogsClient;
 
+        otel_debug!(name: "LogsTonicChannelBuilding");
+
         let (channel, interceptor, compression) = self.build_channel(
             crate::logs::OTEL_EXPORTER_OTLP_LOGS_ENDPOINT,
             crate::logs::OTEL_EXPORTER_OTLP_LOGS_TIMEOUT,
@@ -325,19 +265,20 @@ impl TonicExporterBuilder {
 
         let client = TonicLogsClient::new(channel, interceptor, compression);
 
-        Ok(crate::logs::LogExporter::new(client))
+        Ok(crate::logs::LogExporter::from_tonic(client))
     }
 
     /// Build a new tonic metrics exporter
     #[cfg(feature = "metrics")]
-    pub fn build_metrics_exporter(
+    pub(crate) fn build_metrics_exporter(
         self,
-        aggregation_selector: Box<dyn opentelemetry_sdk::metrics::reader::AggregationSelector>,
-        temporality_selector: Box<dyn opentelemetry_sdk::metrics::reader::TemporalitySelector>,
-    ) -> opentelemetry::metrics::Result<crate::MetricsExporter> {
-        use crate::MetricsExporter;
+        temporality: opentelemetry_sdk::metrics::Temporality,
+    ) -> Result<crate::MetricExporter, ExporterBuildError> {
+        use crate::MetricExporter;
         use metrics::TonicMetricsClient;
 
+        otel_debug!(name: "MetricsTonicChannelBuilding");
+
         let (channel, interceptor, compression) = self.build_channel(
             crate::metric::OTEL_EXPORTER_OTLP_METRICS_ENDPOINT,
             crate::metric::OTEL_EXPORTER_OTLP_METRICS_TIMEOUT,
@@ -347,20 +288,16 @@ impl TonicExporterBuilder {
 
         let client = TonicMetricsClient::new(channel, interceptor, compression);
 
-        Ok(MetricsExporter::new(
-            client,
-            temporality_selector,
-            aggregation_selector,
-        ))
+        Ok(MetricExporter::from_tonic(client, temporality))
     }
 
     /// Build a new tonic span exporter
     #[cfg(feature = "trace")]
-    pub fn build_span_exporter(
-        self,
-    ) -> Result<crate::SpanExporter, opentelemetry::trace::TraceError> {
+    pub(crate) fn build_span_exporter(self) -> Result<crate::SpanExporter, ExporterBuildError> {
         use crate::exporter::tonic::trace::TonicTracesClient;
 
+        otel_debug!(name: "TracesTonicChannelBuilding");
+
         let (channel, interceptor, compression) = self.build_channel(
             crate::span::OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
             crate::span::OTEL_EXPORTER_OTLP_TRACES_TIMEOUT,
@@ -370,7 +307,7 @@ impl TonicExporterBuilder {
 
         let client = TonicTracesClient::new(channel, interceptor, compression);
 
-        Ok(crate::SpanExporter::new(client))
+        Ok(crate::SpanExporter::from_tonic(client))
     }
 }
 
@@ -388,28 +325,130 @@ fn merge_metadata_with_headers_from_env(
     }
 }
 
-fn parse_headers_from_env(signal_headers_var: &str) -> HeaderMap {
-    env::var(signal_headers_var)
-        .or_else(|_| env::var(OTEL_EXPORTER_OTLP_HEADERS))
-        .map(|input| {
-            parse_header_string(&input)
-                .filter_map(|(key, value)| {
-                    Some((
-                        HeaderName::from_str(key).ok()?,
-                        HeaderValue::from_str(&value).ok()?,
-                    ))
-                })
-                .collect::<HeaderMap>()
-        })
-        .unwrap_or_default()
+fn parse_headers_from_env(signal_headers_var: &str) -> (HeaderMap, Vec<(String, String)>) {
+    let mut headers = Vec::new();
+
+    (
+        env::var(signal_headers_var)
+            .or_else(|_| env::var(OTEL_EXPORTER_OTLP_HEADERS))
+            .map(|input| {
+                parse_header_string(&input)
+                    .filter_map(|(key, value)| {
+                        headers.push((key.to_owned(), value.clone()));
+                        Some((
+                            HeaderName::from_str(key).ok()?,
+                            HeaderValue::from_str(&value).ok()?,
+                        ))
+                    })
+                    .collect::<HeaderMap>()
+            })
+            .unwrap_or_default(),
+        headers,
+    )
+}
+
+/// Expose interface for modifying [TonicConfig] fields within the exporter builders.
+pub trait HasTonicConfig {
+    /// Return a mutable reference to the export config within the exporter builders.
+    fn tonic_config(&mut self) -> &mut TonicConfig;
+}
+
+/// Expose interface for modifying [TonicConfig] fields within the [TonicExporterBuilder].
+impl HasTonicConfig for TonicExporterBuilder {
+    fn tonic_config(&mut self) -> &mut TonicConfig {
+        &mut self.tonic_config
+    }
+}
+
+/// Expose methods to override [TonicConfig].
+///
+/// This trait will be implemented for every struct that implemented [`HasTonicConfig`] trait.
+///
+/// ## Examples
+/// ```
+/// # #[cfg(all(feature = "trace", feature = "grpc-tonic"))]
+/// # {
+/// use crate::opentelemetry_otlp::{WithExportConfig, WithTonicConfig};
+/// let exporter_builder = opentelemetry_otlp::SpanExporter::builder()
+///     .with_tonic()
+///     .with_compression(opentelemetry_otlp::Compression::Gzip);
+/// # }
+/// ```
+pub trait WithTonicConfig {
+    /// Set the TLS settings for the collector endpoint.
+    #[cfg(feature = "tls")]
+    fn with_tls_config(self, tls_config: ClientTlsConfig) -> Self;
+
+    /// Set custom metadata entries to send to the collector.
+    fn with_metadata(self, metadata: MetadataMap) -> Self;
+
+    /// Set the compression algorithm to use when communicating with the collector.
+    fn with_compression(self, compression: Compression) -> Self;
+
+    /// Use `channel` as tonic's transport channel.
+    /// this will override tls config and should only be used
+    /// when working with non-HTTP transports.
+    ///
+    /// Users MUST make sure the [`ExportConfig::timeout`] is
+    /// the same as the channel's timeout.
+    fn with_channel(self, channel: tonic::transport::Channel) -> Self;
+
+    /// Use a custom `interceptor` to modify each outbound request.
+    /// this can be used to modify the grpc metadata, for example
+    /// to inject auth tokens.
+    fn with_interceptor<I>(self, interceptor: I) -> Self
+    where
+        I: tonic::service::Interceptor + Clone + Send + Sync + 'static;
+}
+
+impl<B: HasTonicConfig> WithTonicConfig for B {
+    #[cfg(feature = "tls")]
+    fn with_tls_config(mut self, tls_config: ClientTlsConfig) -> Self {
+        self.tonic_config().tls_config = Some(tls_config);
+        self
+    }
+
+    /// Set custom metadata entries to send to the collector.
+    fn with_metadata(mut self, metadata: MetadataMap) -> Self {
+        // extending metadata maps is harder than just casting back/forth
+        let mut existing_headers = self
+            .tonic_config()
+            .metadata
+            .clone()
+            .unwrap_or_default()
+            .into_headers();
+        existing_headers.extend(metadata.into_headers());
+
+        self.tonic_config().metadata = Some(MetadataMap::from_headers(existing_headers));
+        self
+    }
+
+    fn with_compression(mut self, compression: Compression) -> Self {
+        self.tonic_config().compression = Some(compression);
+        self
+    }
+
+    fn with_channel(mut self, channel: tonic::transport::Channel) -> Self {
+        self.tonic_config().channel = Some(channel);
+        self
+    }
+
+    fn with_interceptor<I>(mut self, interceptor: I) -> Self
+    where
+        I: tonic::service::Interceptor + Clone + Send + Sync + 'static,
+    {
+        self.tonic_config().interceptor = Some(BoxInterceptor(Box::new(interceptor)));
+        self
+    }
 }
 
 #[cfg(test)]
 mod tests {
     use crate::exporter::tests::run_env_test;
+    use crate::exporter::tonic::WithTonicConfig;
     #[cfg(feature = "grpc-tonic")]
     use crate::exporter::Compression;
-    use crate::TonicExporterBuilder;
+    use crate::{TonicExporterBuilder, OTEL_EXPORTER_OTLP_TRACES_ENDPOINT};
     use crate::{OTEL_EXPORTER_OTLP_HEADERS, OTEL_EXPORTER_OTLP_TRACES_HEADERS};
     use http::{HeaderMap, HeaderName, HeaderValue};
     use tonic::metadata::{MetadataMap, MetadataValue};
@@ -421,7 +460,9 @@ mod tests {
         metadata.insert("foo", "bar".parse().unwrap());
         let builder = TonicExporterBuilder::default().with_metadata(metadata);
         let result = builder.tonic_config.metadata.unwrap();
-        let foo = result.get("foo").unwrap();
+        let foo = result
+            .get("foo")
+            .expect("there to always be an entry for foo");
         assert_eq!(foo, &MetadataValue::try_from("bar").unwrap());
         assert!(result.get("User-Agent").is_some());
 
@@ -462,7 +503,6 @@ mod tests {
     }
 
     #[test]
-    #[cfg(feature = "grpc-tonic")]
     fn test_convert_compression() {
         #[cfg(feature = "gzip-tonic")]
         assert!(tonic::codec::CompressionEncoding::try_from(Compression::Gzip).is_ok());
@@ -474,6 +514,54 @@ mod tests {
         assert!(tonic::codec::CompressionEncoding::try_from(Compression::Zstd).is_err());
     }
 
+    #[test]
+    fn test_priority_of_signal_env_over_generic_env_for_compression() {
+        run_env_test(
+            vec![
+                (crate::OTEL_EXPORTER_OTLP_TRACES_COMPRESSION, "zstd"),
+                (super::OTEL_EXPORTER_OTLP_COMPRESSION, "gzip"),
+            ],
+            || {
+                let builder = TonicExporterBuilder::default();
+
+                let compression = builder
+                    .resolve_compression(crate::OTEL_EXPORTER_OTLP_TRACES_COMPRESSION)
+                    .unwrap();
+                assert_eq!(compression, Some(tonic::codec::CompressionEncoding::Zstd));
+            },
+        );
+    }
+
+    #[test]
+    fn test_priority_of_code_based_config_over_envs_for_compression() {
+        run_env_test(
+            vec![
+                (crate::OTEL_EXPORTER_OTLP_TRACES_COMPRESSION, "gzip"),
+                (super::OTEL_EXPORTER_OTLP_COMPRESSION, "gzip"),
+            ],
+            || {
+                let builder = TonicExporterBuilder::default().with_compression(Compression::Zstd);
+
+                let compression = builder
+                    .resolve_compression(crate::OTEL_EXPORTER_OTLP_TRACES_COMPRESSION)
+                    .unwrap();
+                assert_eq!(compression, Some(tonic::codec::CompressionEncoding::Zstd));
+            },
+        );
+    }
+
+    #[test]
+    fn test_use_default_when_others_missing_for_compression() {
+        run_env_test(vec![], || {
+            let builder = TonicExporterBuilder::default();
+
+            let compression = builder
+                .resolve_compression(crate::OTEL_EXPORTER_OTLP_TRACES_COMPRESSION)
+                .unwrap();
+            assert!(compression.is_none());
+        });
+    }
+
     #[test]
     fn test_parse_headers_from_env() {
         run_env_test(
@@ -483,7 +571,7 @@ mod tests {
             ],
             || {
                 assert_eq!(
-                    super::parse_headers_from_env(OTEL_EXPORTER_OTLP_TRACES_HEADERS),
+                    super::parse_headers_from_env(OTEL_EXPORTER_OTLP_TRACES_HEADERS).0,
                     HeaderMap::from_iter([
                         (
                             HeaderName::from_static("k1"),
@@ -497,7 +585,7 @@ mod tests {
                 );
 
                 assert_eq!(
-                    super::parse_headers_from_env("EMPTY_ENV"),
+                    super::parse_headers_from_env("EMPTY_ENV").0,
                     HeaderMap::from_iter([(
                         HeaderName::from_static("k3"),
                         HeaderValue::from_static("v3")
@@ -520,7 +608,7 @@ mod tests {
                 metadata.insert("k1", "v0".parse().unwrap());
 
                 let result =
-                    super::merge_metadata_with_headers_from_env(metadata, headers_from_env);
+                    super::merge_metadata_with_headers_from_env(metadata, headers_from_env.0);
 
                 assert_eq!(
                     result.get("foo").unwrap(),
@@ -531,4 +619,47 @@ mod tests {
             },
         );
     }
+
+    #[test]
+    fn test_priority_of_signal_env_over_generic_env_for_endpoint() {
+        run_env_test(
+            vec![
+                (OTEL_EXPORTER_OTLP_TRACES_ENDPOINT, "http://localhost:1234"),
+                (super::OTEL_EXPORTER_OTLP_ENDPOINT, "http://localhost:2345"),
+            ],
+            || {
+                let url = TonicExporterBuilder::resolve_endpoint(
+                    OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
+                    None,
+                );
+                assert_eq!(url, "http://localhost:1234");
+            },
+        );
+    }
+
+    #[test]
+    fn test_priority_of_code_based_config_over_envs_for_endpoint() {
+        run_env_test(
+            vec![
+                (OTEL_EXPORTER_OTLP_TRACES_ENDPOINT, "http://localhost:1234"),
+                (super::OTEL_EXPORTER_OTLP_ENDPOINT, "http://localhost:2345"),
+            ],
+            || {
+                let url = TonicExporterBuilder::resolve_endpoint(
+                    OTEL_EXPORTER_OTLP_TRACES_ENDPOINT,
+                    Some("http://localhost:3456".to_string()),
+                );
+                assert_eq!(url, "http://localhost:3456");
+            },
+        );
+    }
+
+    #[test]
+    fn test_use_default_when_others_missing_for_endpoint() {
+        run_env_test(vec![], || {
+            let url =
+                TonicExporterBuilder::resolve_endpoint(OTEL_EXPORTER_OTLP_TRACES_ENDPOINT, None);
+            assert_eq!(url, "http://localhost:4317");
+        });
+    }
 }
